#ifndef HARKTOOL4_IRPROVIDERTSP_HPP
#define HARKTOOL4_IRPROVIDERTSP_HPP

#include <string>
#include "IRProvider.hpp"

namespace harktool4
{

/* @brief TSPListFileからインパルス応答を提供する.
 * TSPListFileにはインパルス応答がそのまま格納されているわけではないので
 * TSP信号から計算によって求める
 */
class IRProviderTSP : public IRProvider
{
public:
    IRProviderTSP(const std::string& tsp_list_file);
    IRProviderTSP(const IRProviderTSP&) = delete;
    IRProviderTSP& operator=(const IRProviderTSP&) = delete;

    virtual std::vector<float> GetIR(size_t src_index, size_t mic_index);
    virtual size_t GetNumSrc();
    virtual size_t GetNumMic();
    virtual std::vector<boost::shared_ptr<harkio2::harkconfig::Source>> GetSources();
    virtual boost::shared_ptr<std::vector<unsigned int>> GetSrcRelatedMicIndices();
    static boost::shared_ptr<std::vector<unsigned int>> GetSrcRelatedMicIndices(boost::shared_ptr<harkio2::harkconfig::TSPListFile2_0>);

private:
    class Impl;
    boost::shared_ptr<Impl> impl_;
};

}

#endif // HARKTOOL4_IRPROVIDERTSP_HPP

