#include<iostream>
#include<stdio.h>
#include<stdlib.h>
#include<sndfile.h>
#include<string.h>

// open wave file
SNDFILE *waveopen(char *path, int sfmode, SF_INFO *sfinfo) {

	SNDFILE *fp;

	sfinfo->format = 0;
	if ((fp = sf_open(path, sfmode, sfinfo)) == NULL) {
		perror("sf_open");
		return NULL;
	}
	return fp;
}

// read wave data
int readwave(SNDFILE *fp, SF_INFO *sfinfo, int count, double ***wave) {

	if ((*wave = (double **) calloc(sfinfo->channels, sizeof(double *))) == NULL) {
		perror("calloc");
		return -1;
	}

	//assert(count <= sfinfo->frames);
	for (int i = 0; i < sfinfo->channels; i++) {
		if (((*wave)[i] = (double *) calloc(count, sizeof(double))) == NULL) {
			perror("calloc");
			free(*wave);
			return -1;
		}
	}

	// read normalized [-1.0, 1.0] wave data as double
	double *data;

	if ((data = (double *) calloc(sfinfo->channels, sizeof(double))) == NULL) {
		perror("calloc");
		for (int i = 0; i < sfinfo->channels; i++) {
			free((*wave)[i]);
		}
		free(*wave);
		return -1;
	}

	for (int i = 0; i < count; i++) {
		if (sf_readf_double(fp, data, 1) != 1) {
			printf("cannot read frame=%d\n", i);
			break;
		}
		for (int j = 0; j < sfinfo->channels; j++) {
			(*wave)[j][i] = data[j];
		}
	}

	free(data);

	return 1;
}

// read wave file
int readwavfile(char *fname, double ***wave, int *length, int *ch, int *samplerate) {
	SNDFILE *fp;
	SF_INFO sfinfo;

	sfinfo.format = 0;
	if((fp = sf_open(fname, SFM_READ, &sfinfo)) == NULL){
		return -1;
	}
	int ret = readwave(fp, &sfinfo, sfinfo.frames, wave);
	sf_close(fp);
	*length = sfinfo.frames;
	*ch = sfinfo.channels;
	*samplerate = sfinfo.samplerate;

	return ret;
}

// write wave data
int writewave(SNDFILE *fp, SF_INFO *sfinfo, int count, double **wave) {

	short *data;
	int i, j;

	if ((data = (short *) malloc(sizeof(short) * count * sfinfo->channels)) == NULL) {
		perror("malloc");
		exit(0);
	}

	for (i = 0; i < count; i++)
		for (j = 0; j < sfinfo->channels; j++) {
			data[j + i * sfinfo->channels] = (short) wave[j][i];
		}

	sf_writef_short(fp, data, count);

	free(data);

	return 1;
}

// read wave file
int writewavfile(char *fname, double **wave, int length, int ch, int samplerate) {
	SNDFILE *fp;
	SF_INFO sfinfo;

	sfinfo.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
	sfinfo.samplerate = samplerate;
	sfinfo.channels = ch;
	sfinfo.sections = -1;
	sfinfo.seekable = -1;
	//	sfinfo.frames = length;

	fp = sf_open(fname, SFM_WRITE, &sfinfo);
	writewave(fp, &sfinfo, length, wave);
	sf_close(fp);

	return 1;
}

#if 0
int main(int argc, char *argv[]) {

	SNDFILE *fp;
	int mode = SFM_READ;
	SF_INFO sfinfo;
	short data[16*10];
	short wave[16][10];
	int i, j;

	sfinfo.format = 0;
	fp = sf_open("../Database/SPEECH/ATR_F1_D0_150cm.wav", mode, &sfinfo);

	printf("frame %d\n", sfinfo.frames );
	printf("fs %d\n", sfinfo.samplerate );
	printf("ch %d\n", sfinfo.channels );
	printf("fmt 0x%x\n", sfinfo.format);
	printf("sections %d\n", sfinfo.sections );
	printf("seekable %d\n", sfinfo.seekable );

	sf_readf_short(fp, data, 10);
	sf_close(fp);

	for( i = 0; i < 10; i++ )
	for( j = 0; j < 16; j++ ) {
		wave[j][i] = data[j+i*16];
	}

	for( i = 0; i < 16; i++ ) {
		printf("Ch %d = ", i );
		for( j = 0; j < 10; j++ ) {
			printf("%3d ", wave[i][j] );
		}
		printf("\n");
	}
}
#endif
